#|
This file is part of Ordo.

Copyright (C) 2025 Ray Miller

This program is free software: you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation, version 3.

This program is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with
this program. If not, see <https://www.gnu.org/licenses/>.
|#
(define-module (ordo core)
  #:use-module (ordo connection)
  #:use-module (ordo context)
  #:use-module (ordo handler)
  #:use-module (ordo inventory)
  #:use-module (ordo logger)
  #:use-module (ordo playbook)
  #:use-module (ordo play)
  #:use-module (ordo task)
  #:use-module ((srfi srfi-26) #:select (cut)))

(define (run-playbook ctx pb)
  (log-msg 'NOTICE "Running playbook: " (playbook-name pb))
  (set-ctx-playbook! ctx pb)
  (for-each (cut run-play ctx <>) (playbook-plays pb)))

(define (run-play ctx p)
  (log-msg 'NOTICE "Running play: " (play-name p))
  (set-ctx-play! ctx p)
  (let ((hosts (resolve-hosts (ctx-inventory ctx) (play-host p))))
    (if (null? hosts)
        (log-msg 'WARN "No hosts matched: " (play-host p))
        (for-each (cut run-host-play ctx p <>) hosts))))

(define (run-host-play ctx p h)
  (log-msg 'NOTICE "Running play on host: " (host-name h))
  (set-ctx-host! ctx h)
  (call-with-connection
   (host-connection h)
   (lambda (conn)
     (dynamic-wind
       (lambda ()
         (set-ctx-connection! ctx conn))
       (lambda ()
         (for-each (cut run-task ctx <>) (play-tasks p))
         (for-each (cut run-handler ctx <>) (play-handlers p)))
       (lambda ()
         (set-ctx-connection! ctx #f))))
   #:sudo? (play-sudo? p)
   #:sudo-user (play-sudo-user p)
   #:sudo-password (play-sudo-password p)))

(define (run-task ctx t)
  (if ((task-pre-condition t) ctx)
      (begin
        (log-msg 'NOTICE "Running task " (task-name t))
        ((task-action t) ctx))
      (log-msg 'NOTICE "Skipping task " (task-name t) ": pre-condition not met")))

(define (run-handler ctx h)
  (when (member (ctx-triggers ctx) (handler-name h))
    (log-msg 'NOTICE "Running handler: " (handler-name h))
    ((handler-action h) ctx)))
