#|
This file is part of Ordo.

Copyright (C) 2025 Ray Miller

This program is free software: you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation, version 3.

This program is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with
this program. If not, see <https://www.gnu.org/licenses/>.
|#

(define-module (ordo playbook)
  #:use-module (ice-9 eval-string)
  #:use-module (ice-9 textual-ports)
  #:use-module (ordo context)
  #:use-module (ordo handler)
  #:use-module (ordo logger)
  #:use-module (ordo play)
  #:use-module (ordo task)
  #:use-module (ordo util keyword-args)
  #:use-module (srfi srfi-1)  ; lists
  #:use-module (srfi srfi-9)  ; records
  #:use-module (srfi srfi-26) ; cut/cute
  #:use-module (srfi srfi-69) ; hash tables
  #:use-module (srfi srfi-71) ; extended let
  #:export (<playbook>
            playbook
            playbook?
            playbook-name
            playbook-vars
            playbook-plays
            load-playbook
            run-playbook))

(define-record-type <playbook>
  (make-playbook name vars plays)
  playbook?
  (name playbook-name)
  (vars playbook-vars)
  (plays playbook-plays))

(define (playbook name . args)
  (let ((plays kwargs (partition play? args)))
    (make-playbook name (alist->hash-table (keyword-arg #:vars kwargs '())) plays)))

(define (load-playbook filename)
  (log-msg 'INFO "Loading playbook " filename)
  (eval-string (call-with-input-file filename get-string-all)
               #:file filename))

(define (run-playbook pb inventory)
  (log-msg 'NOTICE "Running playbook: " (playbook-name pb))
  (parameterize ((*inventory* inventory)
                 (*playbook* pb))
    (for-each run-play (playbook-plays pb))))
